# How to become a contributor and submit your own code

## Contributor License Agreements

We'd love to accept your sample apps and patches! Before we can take them, we
have to jump a couple of legal hurdles.

Please fill out either the individual or corporate Contributor License
Agreement (CLA).

  * If you are an individual writing original source code and you're sure you
    own the intellectual property, then you'll need to sign an [individual CLA]
    (https://developers.google.com/open-source/cla/individual).
  * If you work for a company that wants to allow you to contribute your work,
    then you'll need to sign a [corporate CLA]
    (https://developers.google.com/open-source/cla/corporate).

Follow either of the two links above to access the appropriate CLA and
instructions for how to sign and return it. Once we receive it, we'll
be able to accept your pull requests.

## Contributing A Patch

1. Submit an issue describing your proposed change to the repo in question.
1. The repo owner will respond to your issue promptly.
1. If your proposed change is accepted, and you haven't already done so, sign a
   Contributor License Agreement (see details above).
1. Fork the desired repo, develop and test your code changes.
1. Ensure that your code adheres to the existing style in the sample to which
   you are contributing.
1. Ensure that your code has an appropriate set of unit tests which all pass.
1. Submit a pull request.

## Style

We adhere as much as possible to the [ruby-style-guide][] and make the code
[rubocop][] compliant. Tests are setup to fail if there are style guide
violations.

## Testing

Please make sure all tests pass before sending a patch. This will help us to
approve your change faster.

As a matter of policy the master branch is always passing all tests, and changes
that break tests cannot be accepted. If that's your case reach out and we can
help you get it fixed.

### Running Tests

```
gem install bundler
bundle install
bundle exec rspec
bundle exec rubocop
```

## Auto generated files

Various of the files in this repository are automatically generated by
puppet-codegen. Such files contain a prominent comment warning for its
auto generated origins. However some types, such as JSON or MD, do not allow
embedding comments without breaking the file or causing side effects.

### Changing auto generated files

Of course these files are not perfect there will inevitably be issues with them.
If you find an issue with them there are 2 options:

1. Send a patch to the affected files to us and we'll update the source used to
   generate the file, thus addressing the issue. Note that in this option your
   patch will not be accepted but will be used as a guide to fix the original
   file.

2. Change the file directly in the sources used by puppet-codegen. By taking
   this approach your change will be attributed to you, as you'd be the author
   of the change. If you'd like to take credit for the change this is the
   recommended approach. This approach has the nice side effect to fix all other
   projects that have the same issue at once.

### File list

The list below contains all the files that were automatically generated by
puppet-codegen:

  * .gitignore
  * .rubocop.yml
  * .tools/end2end/data/address.pp
  * .tools/end2end/data/backend_bucket.pp
  * .tools/end2end/data/backend_service.pp
  * .tools/end2end/data/delete_address.pp
  * .tools/end2end/data/delete_backend_bucket.pp
  * .tools/end2end/data/delete_backend_service.pp
  * .tools/end2end/data/delete_disk.pp
  * .tools/end2end/data/delete_firewall.pp
  * .tools/end2end/data/delete_forwarding_rule.pp
  * .tools/end2end/data/delete_global_address.pp
  * .tools/end2end/data/delete_global_forwarding_rule.pp
  * .tools/end2end/data/delete_health_check.pp
  * .tools/end2end/data/delete_http_health_check.pp
  * .tools/end2end/data/delete_https_health_check.pp
  * .tools/end2end/data/delete_image.pp
  * .tools/end2end/data/delete_instance.pp
  * .tools/end2end/data/delete_instance_group.pp
  * .tools/end2end/data/delete_instance_group_manager.pp
  * .tools/end2end/data/delete_instance_template.pp
  * .tools/end2end/data/delete_network.pp
  * .tools/end2end/data/delete_route.pp
  * .tools/end2end/data/delete_snapshot.pp
  * .tools/end2end/data/delete_ssl_certificate.pp
  * .tools/end2end/data/delete_subnetwork.pp
  * .tools/end2end/data/delete_target_http_proxy.pp
  * .tools/end2end/data/delete_target_https_proxy.pp
  * .tools/end2end/data/delete_target_pool.pp
  * .tools/end2end/data/delete_target_ssl_proxy.pp
  * .tools/end2end/data/delete_target_tcp_proxy.pp
  * .tools/end2end/data/delete_url_map.pp
  * .tools/end2end/data/disk.pp
  * .tools/end2end/data/disk_type.pp
  * .tools/end2end/data/firewall.pp
  * .tools/end2end/data/firewall~change1.pp
  * .tools/end2end/data/forwarding_rule.pp
  * .tools/end2end/data/global_address.pp
  * .tools/end2end/data/global_forwarding_rule.pp
  * .tools/end2end/data/health_check.pp
  * .tools/end2end/data/http_health_check.pp
  * .tools/end2end/data/https_health_check.pp
  * .tools/end2end/data/image.pp
  * .tools/end2end/data/instance.pp
  * .tools/end2end/data/instance_group.pp
  * .tools/end2end/data/instance_group_manager.pp
  * .tools/end2end/data/instance_template.pp
  * .tools/end2end/data/license.pp
  * .tools/end2end/data/machine_type.pp
  * .tools/end2end/data/network~auto.pp
  * .tools/end2end/data/network~convert_to_custom.pp
  * .tools/end2end/data/network~custom.pp
  * .tools/end2end/data/network~legacy.pp
  * .tools/end2end/data/region.pp
  * .tools/end2end/data/route.pp
  * .tools/end2end/data/snapshot.pp
  * .tools/end2end/data/ssl_certificate.pp
  * .tools/end2end/data/subnetwork.pp
  * .tools/end2end/data/target_http_proxy.pp
  * .tools/end2end/data/target_https_proxy.pp
  * .tools/end2end/data/target_pool.pp
  * .tools/end2end/data/target_ssl_proxy.pp
  * .tools/end2end/data/target_tcp_proxy.pp
  * .tools/end2end/data/target_vpn_gateway.pp
  * .tools/end2end/data/url_map.pp
  * .tools/end2end/data/vpn_tunnel.pp
  * .tools/end2end/data/zone.pp
  * CHANGELOG.md
  * CONTRIBUTING.md
  * examples/address.pp
  * examples/backend_bucket.pp
  * examples/backend_service.pp
  * examples/delete_address.pp
  * examples/delete_backend_bucket.pp
  * examples/delete_backend_service.pp
  * examples/delete_disk.pp
  * examples/delete_firewall.pp
  * examples/delete_forwarding_rule.pp
  * examples/delete_global_address.pp
  * examples/delete_global_forwarding_rule.pp
  * examples/delete_health_check.pp
  * examples/delete_http_health_check.pp
  * examples/delete_https_health_check.pp
  * examples/delete_image.pp
  * examples/delete_instance.pp
  * examples/delete_instance_group.pp
  * examples/delete_instance_group_manager.pp
  * examples/delete_instance_template.pp
  * examples/delete_network.pp
  * examples/delete_route.pp
  * examples/delete_snapshot.pp
  * examples/delete_ssl_certificate.pp
  * examples/delete_subnetwork.pp
  * examples/delete_target_http_proxy.pp
  * examples/delete_target_https_proxy.pp
  * examples/delete_target_pool.pp
  * examples/delete_target_ssl_proxy.pp
  * examples/delete_target_tcp_proxy.pp
  * examples/delete_url_map.pp
  * examples/disk.pp
  * examples/disk_type.pp
  * examples/firewall.pp
  * examples/firewall~change1.pp
  * examples/forwarding_rule.pp
  * examples/global_address.pp
  * examples/global_forwarding_rule.pp
  * examples/health_check.pp
  * examples/http_health_check.pp
  * examples/https_health_check.pp
  * examples/image.pp
  * examples/instance.pp
  * examples/instance_group.pp
  * examples/instance_group_manager.pp
  * examples/instance_template.pp
  * examples/license.pp
  * examples/machine_type.pp
  * examples/network~auto.pp
  * examples/network~convert_to_custom.pp
  * examples/network~custom.pp
  * examples/network~legacy.pp
  * examples/region.pp
  * examples/route.pp
  * examples/snapshot.pp
  * examples/ssl_certificate.pp
  * examples/subnetwork.pp
  * examples/target_http_proxy.pp
  * examples/target_https_proxy.pp
  * examples/target_pool.pp
  * examples/target_ssl_proxy.pp
  * examples/target_tcp_proxy.pp
  * examples/target_vpn_gateway.pp
  * examples/url_map.pp
  * examples/vpn_tunnel.pp
  * examples/zone.pp
  * Gemfile
  * lib/google/compute/api/gcompute_disk.rb
  * lib/google/compute/api/gcompute_instance.rb
  * lib/google/compute/network/base.rb
  * lib/google/compute/network/delete.rb
  * lib/google/compute/network/get.rb
  * lib/google/compute/network/patch.rb
  * lib/google/compute/network/post.rb
  * lib/google/compute/network/put.rb
  * lib/google/compute/property/address_address.rb
  * lib/google/compute/property/address_address_type.rb
  * lib/google/compute/property/array.rb
  * lib/google/compute/property/backendservice_backends.rb
  * lib/google/compute/property/backendservice_cache_key_policy.rb
  * lib/google/compute/property/backendservice_cdn_policy.rb
  * lib/google/compute/property/backendservice_connection_draining.rb
  * lib/google/compute/property/backendservice_iap.rb
  * lib/google/compute/property/backendservice_selflink.rb
  * lib/google/compute/property/base.rb
  * lib/google/compute/property/boolean.rb
  * lib/google/compute/property/disk_disk_encryption_key.rb
  * lib/google/compute/property/disk_name.rb
  * lib/google/compute/property/disk_selflink.rb
  * lib/google/compute/property/disk_source_image_encryption_key.rb
  * lib/google/compute/property/disk_source_snapshot_encryption_key.rb
  * lib/google/compute/property/disktype_deprecated.rb
  * lib/google/compute/property/disktype_selflink.rb
  * lib/google/compute/property/double.rb
  * lib/google/compute/property/enum.rb
  * lib/google/compute/property/firewall_allowed.rb
  * lib/google/compute/property/firewall_denied.rb
  * lib/google/compute/property/forwardingrule_selflink.rb
  * lib/google/compute/property/health_check_proxy_header.rb
  * lib/google/compute/property/healthcheck_http_health_check.rb
  * lib/google/compute/property/healthcheck_https_health_check.rb
  * lib/google/compute/property/healthcheck_ssl_health_check.rb
  * lib/google/compute/property/healthcheck_tcp_health_check.rb
  * lib/google/compute/property/httphealthcheck_selflink.rb
  * lib/google/compute/property/image_deprecated.rb
  * lib/google/compute/property/image_guest_os_features.rb
  * lib/google/compute/property/image_image_encryption_key.rb
  * lib/google/compute/property/image_raw_disk.rb
  * lib/google/compute/property/image_source_disk_encryption_key.rb
  * lib/google/compute/property/instance_access_configs.rb
  * lib/google/compute/property/instance_alias_ip_ranges.rb
  * lib/google/compute/property/instance_disk_encryption_key.rb
  * lib/google/compute/property/instance_disks.rb
  * lib/google/compute/property/instance_guest_accelerators.rb
  * lib/google/compute/property/instance_initialize_params.rb
  * lib/google/compute/property/instance_network_interfaces.rb
  * lib/google/compute/property/instance_scheduling.rb
  * lib/google/compute/property/instance_selflink.rb
  * lib/google/compute/property/instance_service_accounts.rb
  * lib/google/compute/property/instance_source_image_encryption_key.rb
  * lib/google/compute/property/instance_tags.rb
  * lib/google/compute/property/instancegroup_named_ports.rb
  * lib/google/compute/property/instancegroup_selflink.rb
  * lib/google/compute/property/instancegroupmanager_current_actions.rb
  * lib/google/compute/property/instancegroupmanager_named_ports.rb
  * lib/google/compute/property/instancetemplate_access_configs.rb
  * lib/google/compute/property/instancetemplate_alias_ip_ranges.rb
  * lib/google/compute/property/instancetemplate_disk_encryption_key.rb
  * lib/google/compute/property/instancetemplate_disks.rb
  * lib/google/compute/property/instancetemplate_guest_accelerators.rb
  * lib/google/compute/property/instancetemplate_initialize_params.rb
  * lib/google/compute/property/instancetemplate_network_interfaces.rb
  * lib/google/compute/property/instancetemplate_properties.rb
  * lib/google/compute/property/instancetemplate_scheduling.rb
  * lib/google/compute/property/instancetemplate_selflink.rb
  * lib/google/compute/property/instancetemplate_service_accounts.rb
  * lib/google/compute/property/instancetemplate_source_image_encryption_key.rb
  * lib/google/compute/property/instancetemplate_tags.rb
  * lib/google/compute/property/integer.rb
  * lib/google/compute/property/license_selflink.rb
  * lib/google/compute/property/machinetype_deprecated.rb
  * lib/google/compute/property/machinetype_name.rb
  * lib/google/compute/property/machinetype_selflink.rb
  * lib/google/compute/property/namevalues.rb
  * lib/google/compute/property/network_routing_config.rb
  * lib/google/compute/property/network_selflink.rb
  * lib/google/compute/property/region_deprecated.rb
  * lib/google/compute/property/region_name.rb
  * lib/google/compute/property/region_selflink.rb
  * lib/google/compute/property/router_advertise_mode.rb
  * lib/google/compute/property/router_advertised_ip_ranges.rb
  * lib/google/compute/property/router_bgp.rb
  * lib/google/compute/property/router_selflink.rb
  * lib/google/compute/property/snapshot_selflink.rb
  * lib/google/compute/property/snapshot_snapshot_encryption_key.rb
  * lib/google/compute/property/snapshot_source_disk_encryption_key.rb
  * lib/google/compute/property/sslcertificate_selflink.rb
  * lib/google/compute/property/string.rb
  * lib/google/compute/property/string_array.rb
  * lib/google/compute/property/subnetwork_selflink.rb
  * lib/google/compute/property/targetpool_selflink.rb
  * lib/google/compute/property/targetvpngateway_selflink.rb
  * lib/google/compute/property/time.rb
  * lib/google/compute/property/urlmap_host_rules.rb
  * lib/google/compute/property/urlmap_path_matchers.rb
  * lib/google/compute/property/urlmap_path_rules.rb
  * lib/google/compute/property/urlmap_selflink.rb
  * lib/google/compute/property/urlmap_tests.rb
  * lib/google/compute/property/zone_deprecated.rb
  * lib/google/compute/property/zone_name.rb
  * lib/puppet/functions/gcompute_address_ip.rb
  * lib/puppet/functions/gcompute_address_ref.rb
  * lib/puppet/functions/gcompute_global_address_ref.rb
  * lib/puppet/functions/gcompute_health_check_ref.rb
  * lib/puppet/functions/gcompute_image_family.rb
  * lib/puppet/functions/gcompute_target_http_proxy_ref.rb
  * lib/puppet/functions/gcompute_task_load_params.rb
  * lib/puppet/functions/gcompute_task_validate_param.rb
  * lib/puppet/provider/gcompute_address/google.rb
  * lib/puppet/provider/gcompute_backend_bucket/google.rb
  * lib/puppet/provider/gcompute_backend_service/google.rb
  * lib/puppet/provider/gcompute_disk/google.rb
  * lib/puppet/provider/gcompute_disk_type/google.rb
  * lib/puppet/provider/gcompute_firewall/google.rb
  * lib/puppet/provider/gcompute_forwarding_rule/google.rb
  * lib/puppet/provider/gcompute_global_address/google.rb
  * lib/puppet/provider/gcompute_global_forwarding_rule/google.rb
  * lib/puppet/provider/gcompute_health_check/google.rb
  * lib/puppet/provider/gcompute_http_health_check/google.rb
  * lib/puppet/provider/gcompute_https_health_check/google.rb
  * lib/puppet/provider/gcompute_image/google.rb
  * lib/puppet/provider/gcompute_instance/google.rb
  * lib/puppet/provider/gcompute_instance_group/google.rb
  * lib/puppet/provider/gcompute_instance_group_manager/google.rb
  * lib/puppet/provider/gcompute_instance_template/google.rb
  * lib/puppet/provider/gcompute_license/google.rb
  * lib/puppet/provider/gcompute_machine_type/google.rb
  * lib/puppet/provider/gcompute_network/google.rb
  * lib/puppet/provider/gcompute_region/google.rb
  * lib/puppet/provider/gcompute_route/google.rb
  * lib/puppet/provider/gcompute_router/google.rb
  * lib/puppet/provider/gcompute_snapshot/google.rb
  * lib/puppet/provider/gcompute_ssl_certificate/google.rb
  * lib/puppet/provider/gcompute_subnetwork/google.rb
  * lib/puppet/provider/gcompute_target_http_proxy/google.rb
  * lib/puppet/provider/gcompute_target_https_proxy/google.rb
  * lib/puppet/provider/gcompute_target_pool/google.rb
  * lib/puppet/provider/gcompute_target_ssl_proxy/google.rb
  * lib/puppet/provider/gcompute_target_tcp_proxy/google.rb
  * lib/puppet/provider/gcompute_target_vpn_gateway/google.rb
  * lib/puppet/provider/gcompute_url_map/google.rb
  * lib/puppet/provider/gcompute_vpn_tunnel/google.rb
  * lib/puppet/provider/gcompute_zone/google.rb
  * lib/puppet/type/gcompute_address.rb
  * lib/puppet/type/gcompute_backend_bucket.rb
  * lib/puppet/type/gcompute_backend_service.rb
  * lib/puppet/type/gcompute_disk.rb
  * lib/puppet/type/gcompute_disk_type.rb
  * lib/puppet/type/gcompute_firewall.rb
  * lib/puppet/type/gcompute_forwarding_rule.rb
  * lib/puppet/type/gcompute_global_address.rb
  * lib/puppet/type/gcompute_global_forwarding_rule.rb
  * lib/puppet/type/gcompute_health_check.rb
  * lib/puppet/type/gcompute_http_health_check.rb
  * lib/puppet/type/gcompute_https_health_check.rb
  * lib/puppet/type/gcompute_image.rb
  * lib/puppet/type/gcompute_instance.rb
  * lib/puppet/type/gcompute_instance_group.rb
  * lib/puppet/type/gcompute_instance_group_manager.rb
  * lib/puppet/type/gcompute_instance_template.rb
  * lib/puppet/type/gcompute_license.rb
  * lib/puppet/type/gcompute_machine_type.rb
  * lib/puppet/type/gcompute_network.rb
  * lib/puppet/type/gcompute_region.rb
  * lib/puppet/type/gcompute_route.rb
  * lib/puppet/type/gcompute_router.rb
  * lib/puppet/type/gcompute_snapshot.rb
  * lib/puppet/type/gcompute_ssl_certificate.rb
  * lib/puppet/type/gcompute_subnetwork.rb
  * lib/puppet/type/gcompute_target_http_proxy.rb
  * lib/puppet/type/gcompute_target_https_proxy.rb
  * lib/puppet/type/gcompute_target_pool.rb
  * lib/puppet/type/gcompute_target_ssl_proxy.rb
  * lib/puppet/type/gcompute_target_tcp_proxy.rb
  * lib/puppet/type/gcompute_target_vpn_gateway.rb
  * lib/puppet/type/gcompute_url_map.rb
  * lib/puppet/type/gcompute_vpn_tunnel.rb
  * lib/puppet/type/gcompute_zone.rb
  * metadata.json
  * README.md
  * spec/.rubocop.yml
  * spec/bundle.rb
  * spec/data/network/gcompute_address/success1~name.yaml
  * spec/data/network/gcompute_address/success1~title.yaml
  * spec/data/network/gcompute_address/success2~name.yaml
  * spec/data/network/gcompute_address/success2~title.yaml
  * spec/data/network/gcompute_address/success3~name.yaml
  * spec/data/network/gcompute_address/success3~title.yaml
  * spec/data/network/gcompute_backend_bucket/success1~name.yaml
  * spec/data/network/gcompute_backend_bucket/success1~title.yaml
  * spec/data/network/gcompute_backend_bucket/success2~name.yaml
  * spec/data/network/gcompute_backend_bucket/success2~title.yaml
  * spec/data/network/gcompute_backend_bucket/success3~name.yaml
  * spec/data/network/gcompute_backend_bucket/success3~title.yaml
  * spec/data/network/gcompute_backend_service/success1~name.yaml
  * spec/data/network/gcompute_backend_service/success1~title.yaml
  * spec/data/network/gcompute_backend_service/success2~name.yaml
  * spec/data/network/gcompute_backend_service/success2~title.yaml
  * spec/data/network/gcompute_backend_service/success3~name.yaml
  * spec/data/network/gcompute_backend_service/success3~title.yaml
  * spec/data/network/gcompute_disk/success1~name.yaml
  * spec/data/network/gcompute_disk/success1~title.yaml
  * spec/data/network/gcompute_disk/success2~name.yaml
  * spec/data/network/gcompute_disk/success2~title.yaml
  * spec/data/network/gcompute_disk/success3~name.yaml
  * spec/data/network/gcompute_disk/success3~title.yaml
  * spec/data/network/gcompute_disk_type/success1~name.yaml
  * spec/data/network/gcompute_disk_type/success1~title.yaml
  * spec/data/network/gcompute_disk_type/success2~name.yaml
  * spec/data/network/gcompute_disk_type/success2~title.yaml
  * spec/data/network/gcompute_disk_type/success3~name.yaml
  * spec/data/network/gcompute_disk_type/success3~title.yaml
  * spec/data/network/gcompute_firewall/success1~name.yaml
  * spec/data/network/gcompute_firewall/success1~title.yaml
  * spec/data/network/gcompute_firewall/success2~name.yaml
  * spec/data/network/gcompute_firewall/success2~title.yaml
  * spec/data/network/gcompute_firewall/success3~name.yaml
  * spec/data/network/gcompute_firewall/success3~title.yaml
  * spec/data/network/gcompute_forwarding_rule/success1~name.yaml
  * spec/data/network/gcompute_forwarding_rule/success1~title.yaml
  * spec/data/network/gcompute_forwarding_rule/success2~name.yaml
  * spec/data/network/gcompute_forwarding_rule/success2~title.yaml
  * spec/data/network/gcompute_forwarding_rule/success3~name.yaml
  * spec/data/network/gcompute_forwarding_rule/success3~title.yaml
  * spec/data/network/gcompute_global_address/success1~name.yaml
  * spec/data/network/gcompute_global_address/success1~title.yaml
  * spec/data/network/gcompute_global_address/success2~name.yaml
  * spec/data/network/gcompute_global_address/success2~title.yaml
  * spec/data/network/gcompute_global_address/success3~name.yaml
  * spec/data/network/gcompute_global_address/success3~title.yaml
  * spec/data/network/gcompute_global_forwarding_rule/success1~name.yaml
  * spec/data/network/gcompute_global_forwarding_rule/success1~title.yaml
  * spec/data/network/gcompute_global_forwarding_rule/success2~name.yaml
  * spec/data/network/gcompute_global_forwarding_rule/success2~title.yaml
  * spec/data/network/gcompute_global_forwarding_rule/success3~name.yaml
  * spec/data/network/gcompute_global_forwarding_rule/success3~title.yaml
  * spec/data/network/gcompute_health_check/success1~name.yaml
  * spec/data/network/gcompute_health_check/success1~title.yaml
  * spec/data/network/gcompute_health_check/success2~name.yaml
  * spec/data/network/gcompute_health_check/success2~title.yaml
  * spec/data/network/gcompute_health_check/success3~name.yaml
  * spec/data/network/gcompute_health_check/success3~title.yaml
  * spec/data/network/gcompute_http_health_check/success1~name.yaml
  * spec/data/network/gcompute_http_health_check/success1~title.yaml
  * spec/data/network/gcompute_http_health_check/success2~name.yaml
  * spec/data/network/gcompute_http_health_check/success2~title.yaml
  * spec/data/network/gcompute_http_health_check/success3~name.yaml
  * spec/data/network/gcompute_http_health_check/success3~title.yaml
  * spec/data/network/gcompute_https_health_check/success1~name.yaml
  * spec/data/network/gcompute_https_health_check/success1~title.yaml
  * spec/data/network/gcompute_https_health_check/success2~name.yaml
  * spec/data/network/gcompute_https_health_check/success2~title.yaml
  * spec/data/network/gcompute_https_health_check/success3~name.yaml
  * spec/data/network/gcompute_https_health_check/success3~title.yaml
  * spec/data/network/gcompute_image/success1~name.yaml
  * spec/data/network/gcompute_image/success1~title.yaml
  * spec/data/network/gcompute_image/success2~name.yaml
  * spec/data/network/gcompute_image/success2~title.yaml
  * spec/data/network/gcompute_image/success3~name.yaml
  * spec/data/network/gcompute_image/success3~title.yaml
  * spec/data/network/gcompute_instance/success1~name.yaml
  * spec/data/network/gcompute_instance/success1~title.yaml
  * spec/data/network/gcompute_instance/success2~name.yaml
  * spec/data/network/gcompute_instance/success2~title.yaml
  * spec/data/network/gcompute_instance/success3~name.yaml
  * spec/data/network/gcompute_instance/success3~title.yaml
  * spec/data/network/gcompute_instance_group/success1~name.yaml
  * spec/data/network/gcompute_instance_group/success1~title.yaml
  * spec/data/network/gcompute_instance_group/success2~name.yaml
  * spec/data/network/gcompute_instance_group/success2~title.yaml
  * spec/data/network/gcompute_instance_group/success3~name.yaml
  * spec/data/network/gcompute_instance_group/success3~title.yaml
  * spec/data/network/gcompute_instance_group_manager/success1~name.yaml
  * spec/data/network/gcompute_instance_group_manager/success1~title.yaml
  * spec/data/network/gcompute_instance_group_manager/success2~name.yaml
  * spec/data/network/gcompute_instance_group_manager/success2~title.yaml
  * spec/data/network/gcompute_instance_group_manager/success3~name.yaml
  * spec/data/network/gcompute_instance_group_manager/success3~title.yaml
  * spec/data/network/gcompute_instance_template/success1~name.yaml
  * spec/data/network/gcompute_instance_template/success1~title.yaml
  * spec/data/network/gcompute_instance_template/success2~name.yaml
  * spec/data/network/gcompute_instance_template/success2~title.yaml
  * spec/data/network/gcompute_instance_template/success3~name.yaml
  * spec/data/network/gcompute_instance_template/success3~title.yaml
  * spec/data/network/gcompute_license/success1~name.yaml
  * spec/data/network/gcompute_license/success1~title.yaml
  * spec/data/network/gcompute_license/success2~name.yaml
  * spec/data/network/gcompute_license/success2~title.yaml
  * spec/data/network/gcompute_license/success3~name.yaml
  * spec/data/network/gcompute_license/success3~title.yaml
  * spec/data/network/gcompute_machine_type/success1~name.yaml
  * spec/data/network/gcompute_machine_type/success1~title.yaml
  * spec/data/network/gcompute_machine_type/success2~name.yaml
  * spec/data/network/gcompute_machine_type/success2~title.yaml
  * spec/data/network/gcompute_machine_type/success3~name.yaml
  * spec/data/network/gcompute_machine_type/success3~title.yaml
  * spec/data/network/gcompute_network/success1~name.yaml
  * spec/data/network/gcompute_network/success1~title.yaml
  * spec/data/network/gcompute_network/success2~name.yaml
  * spec/data/network/gcompute_network/success2~title.yaml
  * spec/data/network/gcompute_network/success3~name.yaml
  * spec/data/network/gcompute_network/success3~title.yaml
  * spec/data/network/gcompute_region/success1~name.yaml
  * spec/data/network/gcompute_region/success1~title.yaml
  * spec/data/network/gcompute_region/success2~name.yaml
  * spec/data/network/gcompute_region/success2~title.yaml
  * spec/data/network/gcompute_region/success3~name.yaml
  * spec/data/network/gcompute_region/success3~title.yaml
  * spec/data/network/gcompute_route/success1~name.yaml
  * spec/data/network/gcompute_route/success1~title.yaml
  * spec/data/network/gcompute_route/success2~name.yaml
  * spec/data/network/gcompute_route/success2~title.yaml
  * spec/data/network/gcompute_route/success3~name.yaml
  * spec/data/network/gcompute_route/success3~title.yaml
  * spec/data/network/gcompute_router/success1~name.yaml
  * spec/data/network/gcompute_router/success1~title.yaml
  * spec/data/network/gcompute_router/success2~name.yaml
  * spec/data/network/gcompute_router/success2~title.yaml
  * spec/data/network/gcompute_router/success3~name.yaml
  * spec/data/network/gcompute_router/success3~title.yaml
  * spec/data/network/gcompute_snapshot/success1~name.yaml
  * spec/data/network/gcompute_snapshot/success1~title.yaml
  * spec/data/network/gcompute_snapshot/success2~name.yaml
  * spec/data/network/gcompute_snapshot/success2~title.yaml
  * spec/data/network/gcompute_snapshot/success3~name.yaml
  * spec/data/network/gcompute_snapshot/success3~title.yaml
  * spec/data/network/gcompute_ssl_certificate/success1~name.yaml
  * spec/data/network/gcompute_ssl_certificate/success1~title.yaml
  * spec/data/network/gcompute_ssl_certificate/success2~name.yaml
  * spec/data/network/gcompute_ssl_certificate/success2~title.yaml
  * spec/data/network/gcompute_ssl_certificate/success3~name.yaml
  * spec/data/network/gcompute_ssl_certificate/success3~title.yaml
  * spec/data/network/gcompute_subnetwork/success1~name.yaml
  * spec/data/network/gcompute_subnetwork/success1~title.yaml
  * spec/data/network/gcompute_subnetwork/success2~name.yaml
  * spec/data/network/gcompute_subnetwork/success2~title.yaml
  * spec/data/network/gcompute_subnetwork/success3~name.yaml
  * spec/data/network/gcompute_subnetwork/success3~title.yaml
  * spec/data/network/gcompute_target_http_proxy/success1~name.yaml
  * spec/data/network/gcompute_target_http_proxy/success1~title.yaml
  * spec/data/network/gcompute_target_http_proxy/success2~name.yaml
  * spec/data/network/gcompute_target_http_proxy/success2~title.yaml
  * spec/data/network/gcompute_target_http_proxy/success3~name.yaml
  * spec/data/network/gcompute_target_http_proxy/success3~title.yaml
  * spec/data/network/gcompute_target_https_proxy/success1~name.yaml
  * spec/data/network/gcompute_target_https_proxy/success1~title.yaml
  * spec/data/network/gcompute_target_https_proxy/success2~name.yaml
  * spec/data/network/gcompute_target_https_proxy/success2~title.yaml
  * spec/data/network/gcompute_target_https_proxy/success3~name.yaml
  * spec/data/network/gcompute_target_https_proxy/success3~title.yaml
  * spec/data/network/gcompute_target_pool/success1~name.yaml
  * spec/data/network/gcompute_target_pool/success1~title.yaml
  * spec/data/network/gcompute_target_pool/success2~name.yaml
  * spec/data/network/gcompute_target_pool/success2~title.yaml
  * spec/data/network/gcompute_target_pool/success3~name.yaml
  * spec/data/network/gcompute_target_pool/success3~title.yaml
  * spec/data/network/gcompute_target_ssl_proxy/success1~name.yaml
  * spec/data/network/gcompute_target_ssl_proxy/success1~title.yaml
  * spec/data/network/gcompute_target_ssl_proxy/success2~name.yaml
  * spec/data/network/gcompute_target_ssl_proxy/success2~title.yaml
  * spec/data/network/gcompute_target_ssl_proxy/success3~name.yaml
  * spec/data/network/gcompute_target_ssl_proxy/success3~title.yaml
  * spec/data/network/gcompute_target_tcp_proxy/success1~name.yaml
  * spec/data/network/gcompute_target_tcp_proxy/success1~title.yaml
  * spec/data/network/gcompute_target_tcp_proxy/success2~name.yaml
  * spec/data/network/gcompute_target_tcp_proxy/success2~title.yaml
  * spec/data/network/gcompute_target_tcp_proxy/success3~name.yaml
  * spec/data/network/gcompute_target_tcp_proxy/success3~title.yaml
  * spec/data/network/gcompute_target_vpn_gateway/success1~name.yaml
  * spec/data/network/gcompute_target_vpn_gateway/success1~title.yaml
  * spec/data/network/gcompute_target_vpn_gateway/success2~name.yaml
  * spec/data/network/gcompute_target_vpn_gateway/success2~title.yaml
  * spec/data/network/gcompute_target_vpn_gateway/success3~name.yaml
  * spec/data/network/gcompute_target_vpn_gateway/success3~title.yaml
  * spec/data/network/gcompute_url_map/success1~name.yaml
  * spec/data/network/gcompute_url_map/success1~title.yaml
  * spec/data/network/gcompute_url_map/success2~name.yaml
  * spec/data/network/gcompute_url_map/success2~title.yaml
  * spec/data/network/gcompute_url_map/success3~name.yaml
  * spec/data/network/gcompute_url_map/success3~title.yaml
  * spec/data/network/gcompute_vpn_tunnel/success1~name.yaml
  * spec/data/network/gcompute_vpn_tunnel/success1~title.yaml
  * spec/data/network/gcompute_vpn_tunnel/success2~name.yaml
  * spec/data/network/gcompute_vpn_tunnel/success2~title.yaml
  * spec/data/network/gcompute_vpn_tunnel/success3~name.yaml
  * spec/data/network/gcompute_vpn_tunnel/success3~title.yaml
  * spec/data/network/gcompute_zone/success1~name.yaml
  * spec/data/network/gcompute_zone/success1~title.yaml
  * spec/data/network/gcompute_zone/success2~name.yaml
  * spec/data/network/gcompute_zone/success2~title.yaml
  * spec/data/network/gcompute_zone/success3~name.yaml
  * spec/data/network/gcompute_zone/success3~title.yaml
  * spec/fake_auth.rb
  * spec/gcompute_address_provider_spec.rb
  * spec/gcompute_backend_bucket_provider_spec.rb
  * spec/gcompute_backend_service_provider_spec.rb
  * spec/gcompute_disk_provider_spec.rb
  * spec/gcompute_disk_type_provider_spec.rb
  * spec/gcompute_firewall_provider_spec.rb
  * spec/gcompute_forwarding_rule_provider_spec.rb
  * spec/gcompute_global_address_provider_spec.rb
  * spec/gcompute_global_forwarding_rule_provider_spec.rb
  * spec/gcompute_health_check_provider_spec.rb
  * spec/gcompute_http_health_check_provider_spec.rb
  * spec/gcompute_https_health_check_provider_spec.rb
  * spec/gcompute_image_provider_spec.rb
  * spec/gcompute_instance_group_manager_provider_spec.rb
  * spec/gcompute_instance_group_provider_spec.rb
  * spec/gcompute_instance_provider_spec.rb
  * spec/gcompute_instance_template_provider_spec.rb
  * spec/gcompute_license_provider_spec.rb
  * spec/gcompute_machine_type_provider_spec.rb
  * spec/gcompute_network_provider_spec.rb
  * spec/gcompute_region_provider_spec.rb
  * spec/gcompute_route_provider_spec.rb
  * spec/gcompute_router_provider_spec.rb
  * spec/gcompute_snapshot_provider_spec.rb
  * spec/gcompute_ssl_certificate_provider_spec.rb
  * spec/gcompute_subnetwork_provider_spec.rb
  * spec/gcompute_target_http_proxy_provider_spec.rb
  * spec/gcompute_target_https_proxy_provider_spec.rb
  * spec/gcompute_target_pool_provider_spec.rb
  * spec/gcompute_target_ssl_proxy_provider_spec.rb
  * spec/gcompute_target_tcp_proxy_provider_spec.rb
  * spec/gcompute_target_vpn_gateway_provider_spec.rb
  * spec/gcompute_url_map_provider_spec.rb
  * spec/gcompute_vpn_tunnel_provider_spec.rb
  * spec/gcompute_zone_provider_spec.rb
  * spec/network_blocker.rb
  * spec/network_blocker_spec.rb
  * spec/network_delete_spec.rb
  * spec/network_get_spec.rb
  * spec/network_patch_spec.rb
  * spec/network_post_spec.rb
  * spec/network_put_spec.rb
  * spec/puppetlint_spec.rb
  * spec/spec_helper.rb
  * spec/test_constants.rb
  * tasks/instance.json
  * tasks/instance.sh
  * tasks/README.md
  * tasks/reset.json
  * tasks/reset.rb
  * tasks/snapshot.json
  * tasks/snapshot.rb

The list below contains all the files that were automatically sourced from a
central location:

  * .tools/README.md
  * Gemfile.lock
  * lib/google/hash_utils.rb
  * lib/google/object_store.rb
  * lib/google/string_utils.rb
  * LICENSE
  * spec/copyright.rb
  * spec/copyright_spec.rb
  * spec/data/copyright_bad1.rb
  * spec/data/copyright_bad2.rb
  * spec/data/copyright_good1.rb
  * spec/data/copyright_good2.rb
  * spec/data/poor_example.pp
  * spec/hash_utils_spec.rb
  * spec/string_utils_spec.rb
  * spec/stubs/google/authorization.rb

[ruby-style-guide]: https://github.com/bbatsov/ruby-style-guide
[rubocop]: https://rubocop.readthedocs.io/en/latest/
