# Copyright 2018 Google Inc.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# ----------------------------------------------------------------------------
#
#     ***     AUTO GENERATED CODE    ***    AUTO GENERATED CODE     ***
#
# ----------------------------------------------------------------------------
#
#     This file is automatically generated by Magic Modules and manual
#     changes will be clobbered when the file is regenerated.
#
#     Please read more about how to change this file in README.md and
#     CONTRIBUTING.md located at the root of this package.
#
# ----------------------------------------------------------------------------

require 'google/authorization'
require 'google/compute/network/get'
require 'json'
require 'puppet'

# Returns the IP address associated with the Address managed by a
# `gcompute_address` resource.
#
# Arguments:
#   - name: string
#     the name of the address resource
#   - region: string
#     the region where the address resource is allocated
#   - project: string
#     the project name where resource is allocated
#   - cred: authorization
#     the credential to use to authorize the information request
#
# Examples:
#   - gcompute_address_ip('my-server', 'us-central1', 'myproject', $fn_auth)
#
# The credential parameter should be allocated with a
# `gauth_credential_*_for_function` call.
Puppet::Functions.create_function(:gcompute_address_ip) do
  dispatch :gcompute_address_ip do
    param 'String', :name
    param 'String', :region
    param 'String', :project
    param 'Runtime[ruby, "Google::Authorization"]', :cred
  end

  def gcompute_address_ip(name, region, project, cred)
    get_request = ::Google::Compute::Network::Get.new(
      gcompute_address_self_link(name, region, project), cred
    )
    response = JSON.parse(get_request.send.body)
    response['address']
  end

  def gcompute_address_self_link(name, region, project)
    URI.join(
      'https://www.googleapis.com/compute/v1/',
      "projects/#{project}/",
      "regions/#{region}/",
      "addresses/#{name}"
    )
  end
end
