# Copyright 2018 Google Inc.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# ----------------------------------------------------------------------------
#
#     ***     AUTO GENERATED CODE    ***    AUTO GENERATED CODE     ***
#
# ----------------------------------------------------------------------------
#
#     This file is automatically generated by Magic Modules and manual
#     changes will be clobbered when the file is regenerated.
#
#     Please read more about how to change this file in README.md and
#     CONTRIBUTING.md located at the root of this package.
#
# ----------------------------------------------------------------------------

require 'google/compute/property/disk_selflink'
require 'google/compute/property/enum'
require 'google/compute/property/image_deprecated'
require 'google/compute/property/image_guest_os_features'
require 'google/compute/property/image_image_encryption_key'
require 'google/compute/property/image_raw_disk'
require 'google/compute/property/image_source_disk_encryption_key'
require 'google/compute/property/integer'
require 'google/compute/property/string'
require 'google/compute/property/string_array'
require 'google/compute/property/time'
require 'puppet'

Puppet::Type.newtype(:gcompute_image) do
  @doc = <<-DOC
    Represents an Image resource. Google Compute Engine uses operating system images to create the
    root persistent disks for your instances. You specify an image when you create an instance.
    Images contain a boot loader, an operating system, and a root file system. Linux operating
    system images are also capable of running containers on Compute Engine. Images can be either
    public or custom. Public images are provided and maintained by Google, open-source communities,
    and third-party vendors. By default, all projects have access to these images and can use them
    to create instances. Custom images are available only to your project. You can create a custom
    image from root persistent disks and other images. Then, use the custom image to create an
    instance.
  DOC

  autorequire(:gauth_credential) do
    credential = self[:credential]
    raise "#{ref}: required property 'credential' is missing" if credential.nil?
    [credential]
  end

  ensurable

  newparam :credential do
    desc <<-DESC
      A gauth_credential name to be used to authenticate with Google Cloud
      Platform.
    DESC
  end

  newparam(:project) do
    desc 'A Google Cloud Platform project to manage.'
  end

  newparam(:name, namevar: true) do
    # TODO(nelsona): Make this description to match the key of the object.
    desc 'The name of the Image.'
  end

  newproperty(:archive_size_bytes, parent: Google::Compute::Property::Integer) do
    desc 'Size of the image tar.gz archive stored in Google Cloud Storage (in bytes). (output only)'
  end

  newproperty(:creation_timestamp, parent: Google::Compute::Property::Time) do
    desc 'Creation timestamp in RFC3339 text format. (output only)'
  end

  newproperty(:deprecated, parent: Google::Compute::Property::ImageDeprecated) do
    desc 'The deprecation status associated with this image. (output only)'
  end

  newproperty(:description, parent: Google::Compute::Property::String) do
    desc <<-DOC
      An optional description of this resource. Provide this property when you create the resource.
    DOC
  end

  newproperty(:disk_size_gb, parent: Google::Compute::Property::Integer) do
    desc 'Size of the image when restored onto a persistent disk (in GB).'
  end

  newproperty(:family, parent: Google::Compute::Property::String) do
    desc <<-DOC
      The name of the image family to which this image belongs. You can create disks by specifying
      an image family instead of a specific image name. The image family always returns its latest
      image that is not deprecated. The name of the image family must comply with RFC1035.
    DOC
  end

  newproperty(:guest_os_features, parent: Google::Compute::Property::ImageGuestOsFeaturesArray) do
    desc <<-DOC
      A list of features to enable on the guest OS. Applicable for bootable images only. Currently,
      only one feature can be enabled, VIRTIO_SCSI_MULTIQUEUE, which allows each virtual CPU to
      have its own queue. For Windows images, you can only enable VIRTIO_SCSI_MULTIQUEUE on images
      with driver version 1.2.0.1621 or higher. Linux images with kernel versions 3.17 and higher
      will support VIRTIO_SCSI_MULTIQUEUE. For new Windows images, the server might also populate
      this field with the value WINDOWS, to indicate that this is a Windows image. This value is
      purely informational and does not enable or disable any features.
    DOC
  end

  newproperty(:id, parent: Google::Compute::Property::Integer) do
    desc <<-DOC
      The unique identifier for the resource. This identifier is defined by the server. (output
      only)
    DOC
  end

  newproperty(:image_encryption_key, parent: Google::Compute::Property::ImageImageEncryptionKey) do
    desc <<-DOC
      Encrypts the image using a customer-supplied encryption key. After you encrypt an image with
      a customer-supplied key, you must provide the same key if you use the image later (e.g. to
      create a disk from the image)
    DOC
  end

  newproperty(:licenses, parent: Google::Compute::Property::StringArray) do
    desc 'Any applicable license URI.'
  end

  newproperty(:name, parent: Google::Compute::Property::String) do
    desc <<-DOC
      Name of the resource; provided by the client when the resource is created. The name must be
      1-63 characters long, and comply with RFC1035. Specifically, the name must be 1-63 characters
      long and match the regular expression `[a-z]([-a-z0-9]*[a-z0-9])?` which means the first
      character must be a lowercase letter, and all following characters must be a dash, lowercase
      letter, or digit, except the last character, which cannot be a dash.
    DOC
  end

  newproperty(:raw_disk, parent: Google::Compute::Property::ImageRawDisk) do
    desc 'The parameters of the raw disk image.'
  end

  newproperty(:source_disk, parent: Google::Compute::Property::DiskSelfLinkRef) do
    desc <<-DOC
      Refers to a gcompute_disk object You must provide either this property or the rawDisk.source
      property but not both to create an image.
    DOC
  end

  newproperty(:source_disk_encryption_key,
              parent: Google::Compute::Property::ImageSourceDiskEncryptionKey) do
    desc <<-DOC
      The customer-supplied encryption key of the source disk. Required if the source disk is
      protected by a customer-supplied encryption key.
    DOC
  end

  newproperty(:source_disk_id, parent: Google::Compute::Property::String) do
    desc <<-DOC
      The ID value of the disk used to create this image. This value may be used to determine
      whether the image was taken from the current or a previous instance of a given disk name.
    DOC
  end

  newproperty(:source_type, parent: Google::Compute::Property::Enum) do
    desc 'The type of the image used to create this disk. The default and only value is RAW'
    newvalue(:RAW)
  end
end
