# Copyright 2018 Google Inc.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# ----------------------------------------------------------------------------
#
#     ***     AUTO GENERATED CODE    ***    AUTO GENERATED CODE     ***
#
# ----------------------------------------------------------------------------
#
#     This file is automatically generated by Magic Modules and manual
#     changes will be clobbered when the file is regenerated.
#
#     Please read more about how to change this file in README.md and
#     CONTRIBUTING.md located at the root of this package.
#
# ----------------------------------------------------------------------------

require 'google/compute/property/instancegroup_selflink'
require 'google/compute/property/instancegroupmanager_current_actions'
require 'google/compute/property/instancegroupmanager_named_ports'
require 'google/compute/property/instancetemplate_selflink'
require 'google/compute/property/integer'
require 'google/compute/property/region_selflink'
require 'google/compute/property/string'
require 'google/compute/property/targetpool_selflink'
require 'google/compute/property/time'
require 'google/compute/property/zone_name'
require 'google/object_store'
require 'puppet'

Puppet::Type.newtype(:gcompute_instance_group_manager) do
  @doc = <<-DOC
    Creates a managed instance group using the information that you specify in the request. After
    the group is created, it schedules an action to create instances in the group using the
    specified instance template. This operation is marked as DONE when the group is created even if
    the instances in the group have not yet been created. You must separately verify the status of
    the individual instances. A managed instance group can have up to 1000 VM instances per group.
  DOC

  autorequire(:gauth_credential) do
    credential = self[:credential]
    raise "#{ref}: required property 'credential' is missing" if credential.nil?
    [credential]
  end

  autorequire(:gcompute_zone) do
    reference = self[:zone]
    raise "#{ref} required property 'zone' is missing" if reference.nil?
    reference.autorequires
  end

  ensurable

  newparam :credential do
    desc <<-DESC
      A gauth_credential name to be used to authenticate with Google Cloud
      Platform.
    DESC
  end

  newparam(:project) do
    desc 'A Google Cloud Platform project to manage.'
  end

  newparam(:name, namevar: true) do
    # TODO(nelsona): Make this description to match the key of the object.
    desc 'The name of the InstanceGroupManager.'
  end

  newparam(:zone, parent: Google::Compute::Property::ZoneNameRef) do
    desc 'The zone the managed instance group resides.'
  end

  newproperty(:base_instance_name, parent: Google::Compute::Property::String) do
    desc <<-DOC
      The base instance name to use for instances in this group. The value must be 1-58 characters
      long. Instances are named by appending a hyphen and a random four-character string to the
      base instance name. The base instance name must comply with RFC1035.
    DOC
  end

  newproperty(:creation_timestamp, parent: Google::Compute::Property::Time) do
    desc <<-DOC
      The creation timestamp for this managed instance group in RFC3339 text format. (output only)
    DOC
  end

  newproperty(:current_actions,
              parent: Google::Compute::Property::InstanceGroupManagerCurrentActions) do
    desc <<-DOC
      The list of instance actions and the number of instances in this managed instance group that
      are scheduled for each of those actions. (output only)
    DOC
  end

  newproperty(:description, parent: Google::Compute::Property::String) do
    desc <<-DOC
      An optional description of this resource. Provide this property when you create the resource.
    DOC
  end

  newproperty(:id, parent: Google::Compute::Property::Integer) do
    desc 'A unique identifier for this resource (output only)'
  end

  newproperty(:instance_group, parent: Google::Compute::Property::InstanceGroupSelfLinkRef) do
    desc 'The instance group being managed (output only)'
  end

  newproperty(:instance_template, parent: Google::Compute::Property::InstanceTemplateSelfLinkRef) do
    desc <<-DOC
      The instance template that is specified for this managed instance group. The group uses this
      template to create all new instances in the managed instance group.
    DOC
  end

  newproperty(:name, parent: Google::Compute::Property::String) do
    desc <<-DOC
      The name of the managed instance group. The name must be 1-63 characters long, and comply
      with RFC1035.
    DOC
  end

  newproperty(:named_ports,
              parent: Google::Compute::Property::InstanceGroupManagerNamedPortsArray) do
    desc <<-DOC
      Named ports configured for the Instance Groups complementary to this Instance Group Manager.
    DOC
  end

  newproperty(:region, parent: Google::Compute::Property::RegionSelfLinkRef) do
    desc 'The region this managed instance group resides (for regional resources). (output only)'
  end

  newproperty(:target_pools, parent: Google::Compute::Property::TargetPoolSelfLinkRefArray) do
    desc <<-DOC
      TargetPool resources to which instances in the instanceGroup field are added. The target
      pools automatically apply to all of the instances in the managed instance group.
    DOC
  end

  newproperty(:target_size, parent: Google::Compute::Property::Integer) do
    desc <<-DOC
      The target number of running instances for this managed instance group. Deleting or
      abandoning instances reduces this number. Resizing the group changes this number.
    DOC
  end

  # Returns all properties that a provider can export to other resources
  def exports
    provider.exports
  end
end
